package cn.com.taiji.tile.service.impl;

import cn.com.taiji.common.domain.PageResult;
import cn.com.taiji.common.model.LayerGroupDsView;
import cn.com.taiji.common.model.LayerStyleView;
import cn.com.taiji.cql.model.GeoRequestParam;
import cn.com.taiji.cql.service.IECqlService;
import cn.com.taiji.tile.model.POI;
import cn.com.taiji.tile.service.ITileService;
import cn.com.taiji.tile.style.ShapeStyle;
import cn.com.taiji.tile.util.ShapeUtils;
import lombok.extern.slf4j.Slf4j;
import org.geotools.jdbc.JDBCDataStoreFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;

@Slf4j
@Service
public class TileServiceImpl implements ITileService {

    @Autowired
    private IECqlService ecqlService;


    @Override
    public byte[] tileData(GeoRequestParam requestParam, List<LayerGroupDsView> layerGroupDsViewList,Map<String,List<LayerStyleView>> dataMaps) {

        for(LayerGroupDsView layerGroupDsView:layerGroupDsViewList){


            //获取图层代码对应的样式
            List<LayerStyleView> layerStyleViewList = new ArrayList<LayerStyleView>();

            if(dataMaps.containsKey(layerGroupDsView.getLayerCode())){

                layerStyleViewList = dataMaps.get(layerGroupDsView.getLayerCode());

            }


            //2根据cql规则获取图层对应的空间地理数据

            long currentTimeMillis = System.currentTimeMillis();

            StringBuffer filterCqlStr = new StringBuffer();

            String filterCql = requestParam.getCqlFilter();

            if(filterCql !=null && !"".equals(filterCql)){

                filterCqlStr.append(filterCql);
            }

            String bbox = requestParam.getBbox();

            if(bbox !=null && !"".equals(bbox)){

                StringBuffer sbBbox = new StringBuffer();
                sbBbox.append("(bbox (");
                sbBbox.append(layerGroupDsView.getBoxColumn()+",");
                sbBbox.append(requestParam.getBbox()+"))");

                if(filterCqlStr.length() >0){

                    filterCqlStr.append(" and "+sbBbox.toString());
                }else{

                    filterCqlStr.append(sbBbox.toString());

                }

            }

            String layerDataType = layerGroupDsView.getLayerDataType();
            String layerDataRun = layerGroupDsView.getLayerDataRun();

            if(layerDataType !=null && !"".equals(layerDataType) && "cql".equals(layerDataType)){

                if(layerDataRun !=null && !"".equals(layerDataRun)){

                    if(filterCqlStr.length() >0){

                        filterCqlStr.append(" and "+layerDataRun);
                    }else{

                        filterCqlStr.append(layerDataRun);

                    }

                }

            }

            System.out.println("查询条件:" + filterCqlStr.toString());

            String host = layerGroupDsView.getHost().split(",")[0].split(":")[0];
            String port = layerGroupDsView.getHost().split(",")[0].split(":")[1];

            Map<String, Object> cqlDsParams = new HashMap<String, Object>();
            cqlDsParams.put(JDBCDataStoreFactory.DATABASE.key, layerGroupDsView.getDataBase());
            cqlDsParams.put(JDBCDataStoreFactory.DBTYPE.key, layerGroupDsView.getDbType());
            cqlDsParams.put(JDBCDataStoreFactory.HOST.key, host);
            cqlDsParams.put(JDBCDataStoreFactory.PORT.key, port);
            cqlDsParams.put(JDBCDataStoreFactory.USER.key, layerGroupDsView.getUserName());
            cqlDsParams.put(JDBCDataStoreFactory.PASSWD.key, layerGroupDsView.getUserPassword());
            cqlDsParams.put("tableName", layerGroupDsView.getTableName());


            Map<String, Object> mapParams = new HashMap<String, Object>();

            if(filterCqlStr.length() >0){

                mapParams.put("cql", filterCqlStr.toString());
            }


            PageResult pageResult = ecqlService.cqlQuery(cqlDsParams, mapParams);

            if (pageResult.getRecordCount() > 0) {

                List<Map<String, Object>> cqlDataList = (List<Map<String, Object>>) pageResult.getDatas();

                //3根据空间地理数据生成图层二进制数据
//            System.out.println("返回的数据:" + cqlDataList.toString());

                System.out.println("数据查询耗时:"+(System.currentTimeMillis()-currentTimeMillis));
                long start = System.currentTimeMillis();
                //根据图层名称在mysql表中查询样式
                ShapeStyle shapeStyle = new ShapeStyle();
                InputStream in = ClassLoader.getSystemResourceAsStream("images/gis-layout-tools_example-axcbypyj.png");
                try {
                    BufferedImage image = ImageIO.read(in);
                    shapeStyle.setHeight(12);
                    shapeStyle.setWidth(12);
                    shapeStyle.setPointMarkerImage(image);
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
                byte[] tileByte = ShapeUtils.getTileByte(requestParam.getBbox(), convert(cqlDataList), shapeStyle);
                long end = System.currentTimeMillis();
                System.out.println("根据数据生成瓦片耗时"+(end-start));
                return tileByte;

            } else {

                log.info("根据cql规则:" + filterCqlStr.toString() + "和图层代码:" + requestParam.getLayers() + " 没有找到图层对应的数据源的空间地理数据");
            }

        }

        return new byte[0];
    }

    public static List<POI> convert(List<Map<String, Object>> maps) {
        List<POI> pois = new ArrayList<>();
        for (Map<String, Object> map : maps) {
            POI poi = new POI();
            poi.setId((String) map.get("id"));
            poi.setName((String) map.get("id"));
            if (Objects.nonNull(map.get("location"))) {
                poi.setShape(map.get("location").toString());
            } else {
                poi.setShape(String.format("POINT(%s %s)", map.get("longitude"), map.get("latitude")));
//            poi.setLat(Double.parseDouble((String)map.get("latitude")));
//            poi.setLon(Double.parseDouble((String)map.get("longitude")));
            }
            poi.setPcolor("rgb(255, 255, 0)");
            pois.add(poi);
        }
        return pois;
    }

    @Override
    public List<?> queryElements(String bbox, Integer type, String layerName) {
        return null;
    }

    public static double[] parseBbox(String bboxString) {
        String[] bboxValuesStr = bboxString.split(",");
        if (bboxValuesStr.length != 4) {
            return new double[0]; // 返回空数组表示无效的 BBOX 格式
        }

        double[] bboxValues = new double[4];
        try {
            for (int i = 0; i < 4; i++) {
                bboxValues[i] = Double.parseDouble(bboxValuesStr[i]);
            }
        } catch (NumberFormatException e) {
            return new double[0]; // 返回空数组表示无效的 BBOX 格式
        }

        return bboxValues;
    }

}
